@@ -8,13 +8,15 @@ use crate::{
         ExternalEditorComponent, HelpComponent,
         InspectCommitComponent, MsgComponent, PullComponent,
         PushComponent, PushTagsComponent, RenameBranchComponent,
-        ResetComponent, StashMsgComponent, TagCommitComponent,
+        ResetComponent, RevisionFilesPopup, StashMsgComponent,
+        TagCommitComponent, TagListComponent,
     },
     input::{Input, InputEvent, InputState},
     keys::{KeyConfig, SharedKeyConfig},
     queue::{Action, InternalEvent, NeedsUpdate, Queue},
+    setup_popups,
     strings::{self, order},
-    tabs::{Revlog, StashList, Stashing, Status},
+    tabs::{FilesTab, Revlog, StashList, Stashing, Status},
     ui::style::{SharedTheme, Theme},
 };
 use anyhow::{bail, Result};
@@ -45,6 +47,7 @@ pub struct App {
     stashmsg_popup: StashMsgComponent,
     inspect_commit_popup: InspectCommitComponent,
     external_editor_popup: ExternalEditorComponent,
+    revision_files_popup: RevisionFilesPopup,
     push_popup: PushComponent,
     push_tags_popup: PushTagsComponent,
     pull_popup: PullComponent,
@@ -52,12 +55,14 @@ pub struct App {
     create_branch_popup: CreateBranchComponent,
     rename_branch_popup: RenameBranchComponent,
     select_branch_popup: BranchListComponent,
+    tags_popup: TagListComponent,
     cmdbar: RefCell<CommandBar>,
     tab: usize,
     revlog: Revlog,
     status_tab: Status,
     stashing_tab: Stashing,
     stashlist_tab: StashList,
+    files_tab: FilesTab,
     queue: Queue,
     theme: SharedTheme,
     key_config: SharedKeyConfig,
@@ -101,6 +106,12 @@ impl App {
                 theme.clone(),
                 key_config.clone(),
             ),
+            revision_files_popup: RevisionFilesPopup::new(
+                &queue,
+                sender,
+                theme.clone(),
+                key_config.clone(),
+            ),
             stashmsg_popup: StashMsgComponent::new(
                 queue.clone(),
                 theme.clone(),
@@ -154,6 +165,11 @@ impl App {
                 theme.clone(),
                 key_config.clone(),
             ),
+            tags_popup: TagListComponent::new(
+                &queue,
+                theme.clone(),
+                key_config.clone(),
+            ),
             do_quit: false,
             cmdbar: RefCell::new(CommandBar::new(
                 theme.clone(),
@@ -188,6 +204,12 @@ impl App {
                 theme.clone(),
                 key_config.clone(),
             ),
+            files_tab: FilesTab::new(
+                sender,
+                &queue,
+                theme.clone(),
+                key_config.clone(),
+            ),
             queue,
             theme,
             key_config,
@@ -222,8 +244,9 @@ impl App {
         match self.tab {
             0 => self.status_tab.draw(f, chunks_main[1])?,
             1 => self.revlog.draw(f, chunks_main[1])?,
-            2 => self.stashing_tab.draw(f, chunks_main[1])?,
-            3 => self.stashlist_tab.draw(f, chunks_main[1])?,
+            2 => self.files_tab.draw(f, chunks_main[1])?,
+            3 => self.stashing_tab.draw(f, chunks_main[1])?,
+            4 => self.stashlist_tab.draw(f, chunks_main[1])?,
             _ => bail!("unknown tab"),
         };
 
@@ -256,6 +279,7 @@ impl App {
                     NeedsUpdate::COMMANDS
                 } else if k == self.key_config.tab_status
                     || k == self.key_config.tab_log
+                    || k == self.key_config.tab_files
                     || k == self.key_config.tab_stashing
                     || k == self.key_config.tab_stashes
                 {
@@ -307,6 +331,7 @@ impl App {
         self.commit.update()?;
         self.status_tab.update()?;
         self.revlog.update()?;
+        self.files_tab.update()?;
         self.stashing_tab.update()?;
         self.stashlist_tab.update()?;
 
@@ -324,12 +349,14 @@ impl App {
 
         self.status_tab.update_git(ev)?;
         self.stashing_tab.update_git(ev)?;
+        self.files_tab.update_git(ev)?;
         self.revlog.update_git(ev)?;
         self.blame_file_popup.update_git(ev)?;
         self.inspect_commit_popup.update_git(ev)?;
         self.push_popup.update_git(ev)?;
         self.push_tags_popup.update_git(ev)?;
         self.pull_popup.update_git(ev)?;
+        self.revision_files_popup.update(ev);
 
         //TODO: better system for this
         // can we simply process the queue here and everyone just uses the queue to schedule a cmd update?
@@ -348,12 +375,14 @@ impl App {
         self.status_tab.anything_pending()
             || self.revlog.any_work_pending()
             || self.stashing_tab.anything_pending()
+            || self.files_tab.anything_pending()
             || self.blame_file_popup.any_work_pending()
             || self.inspect_commit_popup.any_work_pending()
             || self.input.is_state_changing()
             || self.push_popup.any_work_pending()
             || self.push_tags_popup.any_work_pending()
             || self.pull_popup.any_work_pending()
+            || self.revision_files_popup.any_work_pending()
     }
 
     ///
@@ -386,14 +415,40 @@ impl App {
             create_branch_popup,
             rename_branch_popup,
             select_branch_popup,
+            revision_files_popup,
+            tags_popup,
             help,
             revlog,
             status_tab,
+            files_tab,
             stashing_tab,
             stashlist_tab
         ]
     );
 
+    setup_popups!(
+        self,
+        [
+            commit,
+            stashmsg_popup,
+            help,
+            inspect_commit_popup,
+            blame_file_popup,
+            external_editor_popup,
+            tag_commit_popup,
+            select_branch_popup,
+            tags_popup,
+            create_branch_popup,
+            rename_branch_popup,
+            revision_files_popup,
+            push_popup,
+            push_tags_popup,
+            pull_popup,
+            reset,
+            msg
+        ]
+    );
+
     fn check_quit_key(&mut self, ev: Event) -> bool {
         if let Event::Key(e) = ev {
             if e == self.key_config.exit {
@@ -408,6 +463,7 @@ impl App {
         vec![
             &mut self.status_tab,
             &mut self.revlog,
+            &mut self.files_tab,
             &mut self.stashing_tab,
             &mut self.stashlist_tab,
         ]
@@ -429,10 +485,12 @@ impl App {
             self.set_tab(0)?
         } else if k == self.key_config.tab_log {
             self.set_tab(1)?
-        } else if k == self.key_config.tab_stashing {
+        } else if k == self.key_config.tab_files {
             self.set_tab(2)?
-        } else if k == self.key_config.tab_stashes {
+        } else if k == self.key_config.tab_stashing {
             self.set_tab(3)?
+        } else if k == self.key_config.tab_stashes {
+            self.set_tab(4)?
         }
 
         Ok(())
@@ -475,6 +533,9 @@ impl App {
         if flags.contains(NeedsUpdate::COMMANDS) {
             self.update_commands();
         }
+        if flags.contains(NeedsUpdate::BRANCHES) {
+            self.select_branch_popup.update_branches()?;
+        }
 
         Ok(())
     }
@@ -536,11 +597,26 @@ impl App {
             InternalEvent::SelectBranch => {
                 self.select_branch_popup.open()?;
             }
+            InternalEvent::Tags => {
+                self.tags_popup.open()?;
+            }
             InternalEvent::TabSwitch => self.set_tab(0)?,
             InternalEvent::InspectCommit(id, tags) => {
                 self.inspect_commit_popup.open(id, tags)?;
                 flags.insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS)
             }
+            InternalEvent::SelectCommitInRevlog(id) => {
+                if let Err(error) = self.revlog.select_commit(id) {
+                    self.queue.borrow_mut().push_back(
+                        InternalEvent::ShowErrorMsg(
+                            error.to_string(),
+                        ),
+                    )
+                } else {
+                    self.tags_popup.hide();
+                    flags.insert(NeedsUpdate::ALL)
+                }
+            }
             InternalEvent::OpenExternalEditor(path) => {
                 self.input.set_polling(false);
                 self.external_editor_popup.show()?;
@@ -559,6 +635,13 @@ impl App {
                 self.push_tags_popup.push_tags()?;
                 flags.insert(NeedsUpdate::ALL)
             }
+            InternalEvent::StatusLastFileMoved => {
+                self.status_tab.last_file_moved()?;
+            }
+            InternalEvent::OpenFileTree(c) => {
+                self.revision_files_popup.open(c)?;
+                flags.insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS)
+            }
         };
 
         Ok(flags)
@@ -599,6 +682,18 @@ impl App {
                     self.select_branch_popup.update_branches()?;
                 }
             }
+            Action::DeleteTag(tag_name) => {
+                if let Err(error) = sync::delete_tag(CWD, &tag_name) {
+                    self.queue.borrow_mut().push_back(
+                        InternalEvent::ShowErrorMsg(
+                            error.to_string(),
+                        ),
+                    )
+                } else {
+                    flags.insert(NeedsUpdate::ALL);
+                    self.tags_popup.update_tags()?;
+                }
+            }
             Action::ForcePush(branch, force) => self
                 .queue
                 .borrow_mut()
@@ -607,6 +702,10 @@ impl App {
                 self.pull_popup.try_conflict_free_merge(rebase);
                 flags.insert(NeedsUpdate::ALL);
             }
+            Action::AbortMerge => {
+                self.status_tab.abort_merge();
+                flags.insert(NeedsUpdate::ALL);
+            }
         };
 
         Ok(())
@@ -655,59 +754,6 @@ impl App {
         res
     }
 
-    //TODO: make this automatic, i keep forgetting to add popups here
-    fn any_popup_visible(&self) -> bool {
-        self.commit.is_visible()
-            || self.help.is_visible()
-            || self.reset.is_visible()
-            || self.msg.is_visible()
-            || self.stashmsg_popup.is_visible()
-            || self.inspect_commit_popup.is_visible()
-            || self.blame_file_popup.is_visible()
-            || self.external_editor_popup.is_visible()
-            || self.tag_commit_popup.is_visible()
-            || self.create_branch_popup.is_visible()
-            || self.push_popup.is_visible()
-            || self.push_tags_popup.is_visible()
-            || self.pull_popup.is_visible()
-            || self.select_branch_popup.is_visible()
-            || self.rename_branch_popup.is_visible()
-    }
-
-    fn draw_popups<B: Backend>(
-        &self,
-        f: &mut Frame<B>,
-    ) -> Result<()> {
-        let size = Layout::default()
-            .direction(Direction::Vertical)
-            .constraints(
-                [
-                    Constraint::Min(1),
-                    Constraint::Length(self.cmdbar.borrow().height()),
-                ]
-                .as_ref(),
-            )
-            .split(f.size())[0];
-
-        self.commit.draw(f, size)?;
-        self.stashmsg_popup.draw(f, size)?;
-        self.help.draw(f, size)?;
-        self.inspect_commit_popup.draw(f, size)?;
-        self.blame_file_popup.draw(f, size)?;
-        self.external_editor_popup.draw(f, size)?;
-        self.tag_commit_popup.draw(f, size)?;
-        self.select_branch_popup.draw(f, size)?;
-        self.create_branch_popup.draw(f, size)?;
-        self.rename_branch_popup.draw(f, size)?;
-        self.push_popup.draw(f, size)?;
-        self.push_tags_popup.draw(f, size)?;
-        self.pull_popup.draw(f, size)?;
-        self.reset.draw(f, size)?;
-        self.msg.draw(f, size)?;
-
-        Ok(())
-    }
-
     //TODO: make this dynamic
     fn draw_tabs<B: Backend>(&self, f: &mut Frame<B>, r: Rect) {
         let r = r.inner(&Margin {
@@ -718,6 +764,7 @@ impl App {
         let tabs = [
             Span::raw(strings::tab_status(&self.key_config)),
             Span::raw(strings::tab_log(&self.key_config)),
+            Span::raw(strings::tab_files(&self.key_config)),
             Span::raw(strings::tab_stashing(&self.key_config)),
             Span::raw(strings::tab_stashes(&self.key_config)),
         ]
