@@ -1,14 +1,16 @@
 use crate::{
     components::{
         visibility_blocking, CommandBlocking, CommandInfo, Component,
-        DrawableComponent,
+        DrawableComponent, EventState,
     },
     keys::SharedKeyConfig,
     strings,
     ui::{self, style::SharedTheme},
 };
-use anyhow::{anyhow, Result};
-use asyncgit::{sync::utils::repo_work_dir, CWD};
+use anyhow::{anyhow, bail, Result};
+use asyncgit::{
+    sync::utils::get_config_string, sync::utils::repo_work_dir, CWD,
+};
 use crossterm::{
     event::Event,
     terminal::{EnterAlternateScreen, LeaveAlternateScreen},
@@ -20,7 +22,8 @@ use std::{env, io, path::Path, process::Command};
 use tui::{
     backend::Backend,
     layout::Rect,
-    widgets::{Block, BorderType, Borders, Clear, Paragraph, Text},
+    text::{Span, Spans},
+    widgets::{Block, BorderType, Borders, Clear, Paragraph},
     Frame,
 };
 
@@ -55,7 +58,7 @@ impl ExternalEditorComponent {
         };
 
         if !path.exists() {
-            return Err(anyhow!("file not found: {:?}", path));
+            bail!("file not found: {:?}", path);
         }
 
         io::stdout().execute(LeaveAlternateScreen)?;
@@ -63,28 +66,50 @@ impl ExternalEditorComponent {
             io::stdout().execute(EnterAlternateScreen).expect("reset terminal");
         }
 
-        let editor = env::var("GIT_EDITOR")
+        let environment_options = ["GIT_EDITOR", "VISUAL", "EDITOR"];
+
+        let editor = env::var(environment_options[0])
             .ok()
-            .or_else(|| env::var("VISUAL").ok())
-            .or_else(|| env::var("EDITOR").ok())
+            .or_else(|| get_config_string(CWD, "core.editor").ok()?)
+            .or_else(|| env::var(environment_options[1]).ok())
+            .or_else(|| env::var(environment_options[2]).ok())
             .unwrap_or_else(|| String::from("vi"));
 
         // TODO: proper handling arguments containing whitespaces
         // This does not do the right thing if the input is `editor --something "with spaces"`
-        let mut editor = editor.split_whitespace();
 
-        let command = editor.next().ok_or_else(|| {
-            anyhow!("unable to read editor command")
+        // deal with "editor name with spaces" p1 p2 p3
+        // and with "editor_no_spaces" p1 p2 p3
+        // does not address spaces in pn
+        let mut echars = editor.chars().peekable();
+
+        let first_char = *echars.peek().ok_or_else(|| {
+            anyhow!(
+                "editor env variable found empty: {}",
+                environment_options.join(" or ")
+            )
         })?;
+        let command: String = if first_char == '\"' {
+            echars
+                .by_ref()
+                .skip(1)
+                .take_while(|c| *c != '\"')
+                .collect()
+        } else {
+            echars.by_ref().take_while(|c| *c != ' ').collect()
+        };
 
-        let mut editor: Vec<&OsStr> =
-            editor.map(|s| OsStr::new(s)).collect();
+        let remainder_str = echars.collect::<String>();
+        let remainder = remainder_str.split_whitespace();
 
-        editor.push(path.as_os_str());
+        let mut args: Vec<&OsStr> =
+            remainder.map(|s| OsStr::new(s)).collect();
 
-        Command::new(command)
+        args.push(path.as_os_str());
+
+        Command::new(command.clone())
             .current_dir(work_dir)
-            .args(editor)
+            .args(args)
             .status()
             .map_err(|e| anyhow!("\"{}\": {}", command, e))?;
 
@@ -99,22 +124,26 @@ impl DrawableComponent for ExternalEditorComponent {
         _rect: Rect,
     ) -> Result<()> {
         if self.visible {
-            let txt = vec![Text::Raw(
-                strings::msg_opening_editor(&self.key_config).into(),
-            )];
+            let txt = Spans::from(
+                strings::msg_opening_editor(&self.key_config)
+                    .split('\n')
+                    .map(|string| {
+                        Span::raw::<String>(string.to_string())
+                    })
+                    .collect::<Vec<Span>>(),
+            );
 
             let area = ui::centered_rect_absolute(25, 3, f.size());
             f.render_widget(Clear, area);
             f.render_widget(
-                Paragraph::new(txt.iter())
+                Paragraph::new(txt)
                     .block(
                         Block::default()
                             .borders(Borders::ALL)
                             .border_type(BorderType::Thick)
-                            .title_style(self.theme.title(true))
                             .border_style(self.theme.block(true)),
                     )
-                    .style(self.theme.text_danger()),
+                    .style(self.theme.block(true)),
                 area,
             );
         }
@@ -136,12 +165,12 @@ impl Component for ExternalEditorComponent {
         visibility_blocking(self)
     }
 
-    fn event(&mut self, _ev: Event) -> Result<bool> {
+    fn event(&mut self, _ev: Event) -> Result<EventState> {
         if self.visible {
-            return Ok(true);
+            return Ok(EventState::Consumed);
         }
 
-        Ok(false)
+        Ok(EventState::NotConsumed)
     }
 
     fn is_visible(&self) -> bool {
