@@ -1,20 +1,26 @@
 #![forbid(unsafe_code)]
-#![deny(unused_imports)]
-#![deny(unused_must_use)]
-#![deny(dead_code)]
-#![deny(clippy::all)]
+#![deny(
+    unused_imports,
+    unused_must_use,
+    dead_code,
+    unstable_name_collisions,
+    unused_assignments
+)]
+#![deny(clippy::all, clippy::perf, clippy::nursery, clippy::pedantic)]
+#![deny(clippy::filetype_is_file)]
 #![deny(clippy::cargo)]
-#![deny(clippy::pedantic)]
-#![deny(clippy::perf)]
-#![deny(clippy::nursery)]
 #![deny(clippy::unwrap_used)]
 #![deny(clippy::panic)]
 #![deny(clippy::match_like_matches_macro)]
 #![deny(clippy::needless_update)]
 #![allow(clippy::module_name_repetitions)]
 #![allow(clippy::multiple_crate_versions)]
+//TODO:
+// #![deny(clippy::expect_used)]
 
 mod app;
+mod args;
+mod bug_report;
 mod clipboard;
 mod cmdbar;
 mod components;
@@ -29,14 +35,10 @@ mod tabs;
 mod ui;
 mod version;
 
-use crate::app::App;
-use anyhow::{anyhow, bail, Result};
+use crate::{app::App, args::process_cmdline};
+use anyhow::{bail, Result};
 use asyncgit::AsyncNotification;
 use backtrace::Backtrace;
-use clap::{
-    crate_authors, crate_description, crate_name, crate_version,
-    App as ClapApp, Arg,
-};
 use crossbeam_channel::{tick, unbounded, Receiver, Select};
 use crossterm::{
     terminal::{
@@ -50,15 +52,10 @@ use keys::KeyConfig;
 use profiler::Profiler;
 use scopeguard::defer;
 use scopetime::scope_time;
-use simplelog::{Config, LevelFilter, WriteLogger};
 use spinner::Spinner;
 use std::{
-    env, fs,
-    fs::File,
     io::{self, Write},
-    panic,
-    path::PathBuf,
-    process,
+    panic, process,
     time::{Duration, Instant},
 };
 use tui::{
@@ -79,10 +76,6 @@ pub enum QueueEvent {
     InputEvent(InputEvent),
 }
 
-struct CliArgs {
-    theme: PathBuf,
-}
-
 fn main() -> Result<()> {
     let cliargs = process_cmdline()?;
 
@@ -102,7 +95,7 @@ fn main() -> Result<()> {
 
     setup_terminal()?;
     defer! {
-        shutdown_terminal().expect("shutdown failed");
+        shutdown_terminal();
     }
 
     set_panic_handlers()?;
@@ -183,10 +176,19 @@ fn setup_terminal() -> Result<()> {
     Ok(())
 }
 
-fn shutdown_terminal() -> Result<()> {
-    io::stdout().execute(LeaveAlternateScreen)?;
-    disable_raw_mode()?;
-    Ok(())
+fn shutdown_terminal() {
+    let leave_screen =
+        io::stdout().execute(LeaveAlternateScreen).map(|_f| ());
+
+    if let Err(e) = leave_screen {
+        eprintln!("leave_screen failed:\n{}", e);
+    }
+
+    let leave_raw_mode = disable_raw_mode();
+
+    if let Err(e) = leave_raw_mode {
+        eprintln!("leave_raw_mode failed:\n{}", e);
+    }
 }
 
 fn draw<B: Backend>(
@@ -201,7 +203,9 @@ fn draw<B: Backend>(
         if let Err(e) = app.draw(&mut f) {
             log::error!("failed to draw: {:?}", e)
         }
-    })
+    })?;
+
+    Ok(())
 }
 
 fn valid_path() -> Result<bool> {
@@ -247,108 +251,24 @@ fn start_terminal<W: Write>(
     Ok(terminal)
 }
 
-fn get_app_cache_path() -> Result<PathBuf> {
-    let mut path = dirs_next::cache_dir()
-        .ok_or_else(|| anyhow!("failed to find os cache dir."))?;
-
-    path.push("gitui");
-    fs::create_dir_all(&path)?;
-    Ok(path)
-}
-
-fn get_app_config_path() -> Result<PathBuf> {
-    let mut path = if cfg!(target_os = "macos") {
-        dirs_next::home_dir().map(|h| h.join(".config"))
-    } else {
-        dirs_next::config_dir()
-    }
-    .ok_or_else(|| anyhow!("failed to find os config dir."))?;
-
-    path.push("gitui");
-    fs::create_dir_all(&path)?;
-    Ok(path)
-}
-
-fn setup_logging() -> Result<()> {
-    let mut path = get_app_cache_path()?;
-    path.push("gitui.log");
-
-    let _ = WriteLogger::init(
-        LevelFilter::Trace,
-        Config::default(),
-        File::create(path)?,
-    );
-
-    Ok(())
-}
-
-fn process_cmdline() -> Result<CliArgs> {
-    let app = ClapApp::new(crate_name!())
-        .author(crate_authors!())
-        .version(crate_version!())
-        .about(crate_description!())
-        .arg(
-            Arg::with_name("theme")
-                .help("Set the color theme (defaults to theme.ron)")
-                .short("t")
-                .long("theme")
-                .value_name("THEME")
-                .takes_value(true),
-        )
-        .arg(
-            Arg::with_name("logging")
-                .help("Stores logging output into a cache directory")
-                .short("l")
-                .long("logging"),
-        )
-        .arg(
-            Arg::with_name("directory")
-                .help("Set the working directory")
-                .short("d")
-                .long("directory")
-                .takes_value(true),
-        );
-
-    let arg_matches = app.get_matches();
-    if arg_matches.is_present("logging") {
-        setup_logging()?;
-    }
-
-    if arg_matches.is_present("directory") {
-        let directory =
-            arg_matches.value_of("directory").unwrap_or(".");
-        env::set_current_dir(directory)?;
-    }
-    let arg_theme =
-        arg_matches.value_of("theme").unwrap_or("theme.ron");
-    if get_app_config_path()?.join(arg_theme).is_file() {
-        Ok(CliArgs {
-            theme: get_app_config_path()?.join(arg_theme),
-        })
-    } else {
-        Ok(CliArgs {
-            theme: get_app_config_path()?.join("theme.ron"),
-        })
-    }
-}
-
 fn set_panic_handlers() -> Result<()> {
     // regular panic handler
     panic::set_hook(Box::new(|e| {
         let backtrace = Backtrace::new();
+        //TODO: create macro to do both in one
         log::error!("panic: {:?}\ntrace:\n{:?}", e, backtrace);
-        shutdown_terminal().expect("shutdown failed inside panic");
         eprintln!("panic: {:?}\ntrace:\n{:?}", e, backtrace);
+        shutdown_terminal();
     }));
 
     // global threadpool
     rayon_core::ThreadPoolBuilder::new()
         .panic_handler(|e| {
             let backtrace = Backtrace::new();
+            //TODO: create macro to do both in one
             log::error!("panic: {:?}\ntrace:\n{:?}", e, backtrace);
-            shutdown_terminal()
-                .expect("shutdown failed inside panic");
             eprintln!("panic: {:?}\ntrace:\n{:?}", e, backtrace);
+            shutdown_terminal();
             process::abort();
         })
         .num_threads(4)
