@@ -1,4 +1,5 @@
 use super::style::SharedTheme;
+use easy_cast::CastFloat;
 use std::convert::TryFrom;
 use tui::{
     backend::Backend,
@@ -12,16 +13,16 @@ use tui::{
 
 ///
 struct Scrollbar {
-    lines: u16,
+    max: u16,
     pos: u16,
     style_bar: Style,
     style_pos: Style,
 }
 
 impl Scrollbar {
-    fn new(lines: usize, pos: usize) -> Self {
+    fn new(max: usize, pos: usize) -> Self {
         Self {
-            lines: u16::try_from(lines).unwrap_or_default(),
+            max: u16::try_from(max).unwrap_or_default(),
             pos: u16::try_from(pos).unwrap_or_default(),
             style_pos: Style::default(),
             style_bar: Style::default(),
@@ -31,39 +32,40 @@ impl Scrollbar {
 
 impl Widget for Scrollbar {
     fn render(self, area: Rect, buf: &mut Buffer) {
+        if area.height <= 2 {
+            return;
+        }
+
+        if self.max == 0 {
+            return;
+        }
+
         let right = area.right().saturating_sub(1);
         if right <= area.left() {
             return;
         };
 
-        let area = area.inner(&Margin {
-            horizontal: 0,
-            vertical: 1,
-        });
+        let (bar_top, bar_height) = {
+            let scrollbar_area = area.inner(&Margin {
+                horizontal: 0,
+                vertical: 1,
+            });
 
-        if area.height == 0 {
-            return;
-        }
+            (scrollbar_area.top(), scrollbar_area.height)
+        };
 
-        if area.height >= self.lines {
-            return;
-        }
-
-        for y in area.top()..area.bottom() {
+        for y in bar_top..(bar_top + bar_height) {
             buf.set_string(right, y, DOUBLE_VERTICAL, self.style_bar);
         }
 
-        let max_pos = self.lines.saturating_sub(area.height);
-        let progress = f32::from(self.pos) / f32::from(max_pos);
+        let progress = f32::from(self.pos) / f32::from(self.max);
         let progress = if progress > 1.0 { 1.0 } else { progress };
-        let pos = f32::from(area.height) * progress;
+        let pos = f32::from(bar_height) * progress;
 
-        //TODO: any better way for this?
-        #[allow(clippy::cast_sign_loss)]
-        #[allow(clippy::cast_possible_truncation)]
-        let pos = (pos as u16).saturating_sub(1);
+        let pos: u16 = pos.cast_nearest();
+        let pos = pos.saturating_sub(1);
 
-        buf.set_string(right, area.top() + pos, FULL, self.style_pos);
+        buf.set_string(right, bar_top + pos, FULL, self.style_pos);
     }
 }
 
@@ -71,10 +73,10 @@ pub fn draw_scrollbar<B: Backend>(
     f: &mut Frame<B>,
     r: Rect,
     theme: &SharedTheme,
-    lines: usize,
+    max: usize,
     pos: usize,
 ) {
-    let mut widget = Scrollbar::new(lines, pos);
+    let mut widget = Scrollbar::new(max, pos);
     widget.style_pos = theme.scroll_bar_pos();
     f.render_widget(widget, r)
 }
